//---------------------------------------------------------------------------
//
// Copyright (C) 2002 Mika Sderholm.
// All Rights Reserved.
//
//---------------------------------------------------------------------------
/*
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All materials mentioning features or use of this software
   must display the following acknowledgement:
      This product includes software developed by Mika Sderholm.
4. Any commercial use is prohibited.

THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL MIKA SDERHOLM BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
//---------------------------------------------------------------------------
//  For more information contact email: mika.soderholm@luukku.com
//---------------------------------------------------------------------------
//  NOTES
//
//  This software reads audio data from the soundcard and shows it
//  using following instruments:
//
//  *Oscilloscope
//  *Stereo Oscilloscope
//  *Correlation between left and right channel
//  *left and right channel peak
//
//  05.03.2002 Init release
//  06.03.2002 Added Peak meter + bug fixes
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "AudioMeter.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TAMForm *AMForm;

void CALLBACK WaveInProc(HWAVEIN waveOut, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{
   if( uMsg == MM_WIM_DATA )
   {
       AMForm->ProcessInput();
   }
}
//---------------------------------------------------------------------------
__fastcall TAMForm::TAMForm(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TAMForm::ProcessInput()
{
//
//  Prepare data buffer
//
   if( isStart )
   {
       WAVEHDR* in=&WaveHeaders[buff];
       ::waveInUnprepareHeader( hWaveIn, in, sizeof(WAVEHDR) );
//
// Views update
//
       timethread->Process(TimeSerie, (short*)(in->lpData));
       correlationthread->Process(Correlation, (short*)(in->lpData));
       stereothread->Process(Stereo, (short*)(in->lpData));
       peakthread->Process(Peak, (short*)(in->lpData));
//
//  Send next buffer;
//
       buff++;
       if( buff == buffers )
          buff=0;
       SendHeader->dwBufferLength=WaveBufSize;
       SendHeader->dwFlags=0;
       SendHeader->dwLoops=0;
       ::waveInPrepareHeader( hWaveIn, SendHeader, sizeof(WAVEHDR) );
       ::waveInAddBuffer( hWaveIn, SendHeader, sizeof(WAVEHDR));
       if( SendHeader == &WaveHeaders[buffers-1] )
          SendHeader = &WaveHeaders[0];
       else
          SendHeader++;
   }
}

void __fastcall TAMForm::FormCreate(TObject *Sender)
{
   buffers = 2;
   buffer_size = 1024;
   buff = 0;
   waveFormat.wFormatTag = WAVE_FORMAT_PCM;
   waveFormat.nChannels = 2;
   waveFormat.nSamplesPerSec = 44100;
   waveFormat.wBitsPerSample = 16;
   waveFormat.nBlockAlign = waveFormat.wBitsPerSample/8*waveFormat.nChannels;
   waveFormat.nAvgBytesPerSec = waveFormat.nBlockAlign*waveFormat.nSamplesPerSec;
   waveFormat.cbSize = 0;
   timethread = new TimeThread( true, buffer_size );
   correlationthread = new CorrelationThread( true, buffer_size );
   stereothread = new StereoThread( true, buffer_size );
   peakthread = new PeakThread( true, buffer_size );
//
//  Audio buffers
//
   WaveHeaders = new WAVEHDR[buffers];
   WaveBufSize = buffer_size*waveFormat.wBitsPerSample/8*waveFormat.nChannels;
   for( int i = 0; i<buffers; i++ )
   {
       WaveHeaders[i].dwBufferLength = WaveBufSize;
       WaveHeaders[i].lpData = (char *)VirtualAlloc( 0, WaveBufSize, MEM_COMMIT, PAGE_READWRITE );
   }
   if( waveInOpen( &hWaveIn, WAVE_MAPPER, &waveFormat, (DWORD)WaveInProc, (DWORD)this, CALLBACK_FUNCTION ) == MMSYSERR_NOERROR)
   {
       for( int i = 0; i<buffers/2; i++ )
       {
           WaveHeaders[i].dwBufferLength=WaveBufSize;
           WaveHeaders[i].dwFlags=0;
           WaveHeaders[i].dwLoops=0;
           if( waveInPrepareHeader( hWaveIn, &WaveHeaders[i], sizeof(WAVEHDR))== MMSYSERR_NOERROR)
                if( waveInAddBuffer( hWaveIn, &WaveHeaders[i], sizeof(WAVEHDR))!= MMSYSERR_NOERROR)
                {
                    Application->MessageBox( "Error", "Multimedia Error", MB_OK);
                    Application->Terminate();
                }
       }
       if( waveInStart( hWaveIn ) == MMSYSERR_NOERROR )
       {
           SendHeader = &WaveHeaders[buffers/2];
           buff = 0;
           isStart = true;
       }
       else
       {
            Application->MessageBox( "Error", "Multimedia Error", MB_OK);
            Application->Terminate();
       }
   }
   else
   {
        Application->MessageBox( "Error", "Multimedia Error", MB_OK);
        Application->Terminate();
   }
}
//---------------------------------------------------------------------------

void __fastcall TAMForm::FormDestroy(TObject *Sender)
{
//
//  Free resources
//
   isStart = false;
   timethread->Terminate();
   correlationthread->Terminate();
   stereothread->Terminate();
   peakthread->Terminate();
//
//  Closing audio device
//
   MMRESULT res=waveInReset( hWaveIn );
   for( int i = 0; i<buffers; i++ )
   {
       res = ::waveInUnprepareHeader( hWaveIn, &WaveHeaders[i], sizeof(WAVEHDR) );
       while( res == WAVERR_STILLPLAYING )
       {
           Sleep( 200 );
           res = ::waveInUnprepareHeader( hWaveIn, &WaveHeaders[i], sizeof(WAVEHDR) );
       }
   }
   ::waveInClose( hWaveIn );

   delete timethread;
   delete correlationthread;
   delete stereothread;
   delete peakthread;

   for( int i=0; i<buffers; i++ )
       VirtualFree( WaveHeaders[i].lpData, 0, MEM_RELEASE );
   delete [] WaveHeaders;
}
//---------------------------------------------------------------------------

void __fastcall TAMForm::CorrelationPaint(TObject *Sender)
{
   Correlation->Canvas->Brush->Color = clBlack;
   Correlation->Canvas->Pen->Color = clWhite;
   Correlation->Canvas->Rectangle( 0, 0, Correlation->ClientWidth, Correlation->ClientHeight );
}
//---------------------------------------------------------------------------



