//---------------------------------------------------------------------------
//
// Copyright (C) 2002 Mika Sderholm.
// All Rights Reserved.
//
//---------------------------------------------------------------------------
/*
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All materials mentioning features or use of this software
   must display the following acknowledgement:
      This product includes software developed by Mika Sderholm.
4. Any commercial use is prohibited.

THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL MIKA SDERHOLM BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
//---------------------------------------------------------------------------
//  For more information contact email: mika.soderholm@luukku.com
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "CorrelationThread.h"
#include <math>
#pragma package(smart_init)
//---------------------------------------------------------------------------
__fastcall CorrelationThread::CorrelationThread( bool CreateSuspended, int ic )
                             :TThread(CreateSuspended)
{
   count = ic;
}

__fastcall CorrelationThread::~CorrelationThread()
{
}

//---------------------------------------------------------------------------
void __fastcall CorrelationThread::Execute()
{
   while ( !Terminated )
   {
      FillBuffer();
   }
}

void __fastcall CorrelationThread::FillBuffer()
{
   double ksum = 0.0;
   double rqsum = 0.0;
   double lqsum = 0.0;
   short* pdatabuff = databuffer;
   for(int i=0; i<count; i++)
   {
       double r = static_cast<double>(*pdatabuff++);
       double l = static_cast<double>(*pdatabuff++);
       ksum += r * l;
       rqsum += r * r;
       lqsum += l * l;
   }
   double correlation = ksum / sqrt(lqsum * rqsum);
//
//  Oscilloscope
//
   view->Canvas->Lock();
   view->Canvas->Brush->Color = clBlack;
   view->Canvas->FillRect( valuerect );
   int half = view->ClientWidth/2;
   int i = half+correlation*half;
   if( i+8 >= view->ClientWidth )
   {
       i = view->ClientWidth-5;
   }
   valuerect.Left=i-4;
   valuerect.Right=valuerect.Left+8;
   valuerect.Top=5;
   valuerect.Bottom=view->Height-5;
   view->Canvas->Brush->Color=clAqua;
   view->Canvas->FillRect( valuerect );
   view->Canvas->Unlock();
   Suspend();
}
//---------------------------------------------------------------------------
void __fastcall CorrelationThread::Process( TPaintBox* in, short* idata )
{
   databuffer=idata;
   view = in;
   Resume();
}

